@@ -8,7 +8,13 @@
 #include <stdio.h>
 #include <gmp.h>
 #include <mpfr.h>
-#include <mpf2mpfr.h>
+//#include <mpf2mpfr.h>
+
+#ifdef _MSC_VER
+void __GSHandlerCheck(void) {}
+void __security_check_cookie(void) {}
+void __security_cookie(void) {}
+#endif
 
 int main(void) {
    mpfr_t x;
